<?php

namespace Drupal\drupalmoduleupgrader\Plugin\DMU\Converter;

use Drupal\Component\Plugin\PluginManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\drupalmoduleupgrader\Converter\Routing\HookMenu;
use Drupal\drupalmoduleupgrader\Converter\Routing\LinkBinding\LinkBinding;
use Drupal\drupalmoduleupgrader\Converter\Routing\LinkBinding\LocalActionLinkBinding;
use Drupal\drupalmoduleupgrader\Converter\Routing\LinkBinding\LocalTaskLinkBinding;
use Drupal\drupalmoduleupgrader\Converter\Routing\LinkBinding\MenuLinkBinding;
use Drupal\drupalmoduleupgrader\Converter\Routing\LinkIndex;
use Drupal\drupalmoduleupgrader\ConverterBase;
use Drupal\drupalmoduleupgrader\TargetInterface;
use Drupal\drupalmoduleupgrader\Utility\Filter\ContainsLogicFilter;
use Pharborist\DocCommentNode;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * @Converter(
 *  id = "links",
 *  description = @Translation("Converts Drupal 7's hook_menu() links to plugin definitions."),
 *  hook = "menu",
 *  fixme = @Translation("@FIXME
This implementation of hook_menu() cannot be automatically converted because
it contains logic (i.e., branching statements, function calls, object
instantiation, etc.) You will need to convert it manually. Sorry!

For more information on how to convert hook_menu() to Drupal 8's new routing
and linking systems, see https://api.drupal.org/api/drupal/core%21includes%21menu.inc/group/menu/8")
 * )
 */
class Links extends ConverterBase implements ContainerFactoryPluginInterface {

  /**
   * @var \Drupal\Component\Plugin\PluginManagerInterface
   */
  protected $routeConverters;

  /**
   * Constructs a Links object.
   *
   * @param array $configuration
   *   Additional configuration for the plugin.
   * @param string $plugin_id
   *   The plugin ID, will be "Links".
   * @param string $plugin_definition
   *   The plugin definition as derived from the annotations.
   * @param \Drupal\Component\Plugin\PluginManagerInterface $route_converters
   *  The plugin manager for route converters, used by HookMenu.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, PluginManagerInterface $route_converters) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->routeConverters = $route_converters;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('plugin.manager.drupalmoduleupgrader.route')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function convert(TargetInterface $target) {
    // If the hook implementation contains logic, we cannot convert it and
    // that's that. So we'll leave a FIXME and bail out.
    /** @var \Pharborist\Functions\FunctionDeclarationNode $hook */
    $hook = $target->getIndexer('hook')->get('menu')->get(0);
    if ($hook->is(new ContainsLogicFilter)) {
      $hook->setDocComment(DocCommentNode::create($this->pluginDefinition['fixme']));
      $target->getCodeManager()->save($hook);
      return;
    }

    // Links are split out by group because there are separate config files
    // for each link type.
    $links = [
      'menu' => new LinkIndex(),
      'task' => new LinkIndex(),
      'action' => new LinkIndex(),
      'contextual' => new LinkIndex(),
    ];

    $hook_menu = new HookMenu($target, $this->routeConverters);
    foreach ($hook_menu->getSourceRoutes()->getAllLinks() as $path => $source) {
      /** @var LinkBinding $binding */
      $binding = LinkBinding::create($source, $hook_menu->getDestinationRoute($path));

      // Skip if the converter wasn't able to find a destination.
      $destination = $binding->getDestination();
      if (empty($destination)) {
        continue;
      }

      if ($binding instanceof MenuLinkBinding) {
        $links['menu']->addBinding($binding);
      }
      elseif ($binding instanceof LocalTaskLinkBinding) {
        $links['task']->addBinding($binding);
      }
      elseif ($binding instanceof LocalActionLinkBinding) {
        $links['action']->addBinding($binding);
      }
      elseif ($source->isContextualLink()) {
        $links['contextual']->addBinding($binding);
      }
    }

    $links = array_map(function(LinkIndex $index) {
      return $index->build();
    }, $links);

    foreach ($links['contextual'] as $link) {
      $link['group'] = $target->id();
    }

    foreach ($links as $group => $data) {
      if ($data) {
        $this->writeInfo($target, 'links.' . $group, $data);
      }
    }
  }

}
