<?php

namespace Drupal\drupalmoduleupgrader\Plugin\DMU\Rewriter;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\PluginBase;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\drupalmoduleupgrader\RewriterInterface;
use Drupal\drupalmoduleupgrader\Utility\Filter\FieldValueFilter;
use Drupal\drupalmoduleupgrader\Utility\Filter\NodeAssignmentFilter;
use Pharborist\ArrayLookupNode;
use Pharborist\Constants\ConstantNode;
use Pharborist\ExpressionNode;
use Pharborist\Filter;
use Pharborist\Functions\CallNode;
use Pharborist\Functions\EmptyNode;
use Pharborist\Functions\IssetNode;
use Pharborist\Functions\ParameterNode;
use Pharborist\Node;
use Pharborist\Objects\ClassConstantLookupNode;
use Pharborist\Objects\ObjectMethodCallNode;
use Pharborist\Objects\ObjectPropertyNode;
use Pharborist\Operators\AssignNode;
use Pharborist\Operators\BooleanNotNode;
use Pharborist\Parser;
use Pharborist\Types\StringNode;
use Pharborist\Variables\VariableNode;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * @Rewriter(
 *  id = "_rewriter",
 *  deriver = "\Drupal\drupalmoduleupgrader\Plugin\DMU\Rewriter\GenericDeriver"
 * )
 */
class Generic extends PluginBase implements RewriterInterface, ContainerFactoryPluginInterface {

  /**
   * @var \Drupal\drupalmoduleupgrader\Utility\Filter\NodeAssignmentFilter
   */
  protected $isAssigned;

  public function __construct(array $configuration, $plugin_id, $plugin_definition, TranslationInterface $translator) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->stringTranslation = $translator;
    $this->isAssigned = new NodeAssignmentFilter();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('string_translation')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function rewrite(ParameterNode $parameter) {
    // Don't even try to rewrite the function if the parameter is reassigned.
    if ($this->isReassigned($parameter)) {
      $error = $this->t('@function() cannot be parametrically rewritten because @parameter is reassigned.', [
        '@parameter' => $parameter->getName(),
        '@function' => $parameter->getFunction()->getName()->getText(),
      ]);
      throw new \LogicException($error);
    }

    foreach ($this->getExpressions($parameter) as $expr) {
      // If the property cannot be determined, don't even try to rewrite the
      // expression.
      $property = $this->getProperty($expr);
      if (empty($property)) {
        continue;
      }

      // If the expression is the left operand of an assignment, it needs
      // to be replaced with a setter method. In all other cases, it's replaced
      // with a getter.
      if ($expr->is($this->isAssigned)) {
        $assignment = $expr->closest(Filter::isInstanceOf('\Pharborist\Operators\AssignNode'));

        if ($setter = $this->rewriteAsSetter($expr, $property, $assignment)) {
          $assignment->replaceWith($setter);
        }
      }
      elseif ($getter = $this->rewriteAsGetter($expr, $property)) {
        $parent = $expr->parent();

        // If the original expression was wrapped by a call to isset() or
        // empty(), we need to replace it entirely.
        if ($getter instanceof CallNode && ($parent instanceof EmptyNode || $parent instanceof IssetNode)) {
          $empty = $parent;

          // If the isset() or empty() call was negated, reverse that logic.
          $parent = $empty->parent();
          if ($parent instanceof BooleanNotNode) {
            $parent->replaceWith($getter);
          }
          else {
            $empty->replaceWith(BooleanNotNode::fromExpression($getter));
          }
        }
        else {
          $expr->replaceWith($getter);
        }
      }
    }

    // Set the type hint, if one is defined.
    if (isset($this->pluginDefinition['type_hint'])) {
      $parameter->setTypeHint($this->pluginDefinition['type_hint']);

      // If the type hint extends FieldableEntityInterface, rewrite any field
      // lookups (e.g. $node->body[LANGUAGE_NONE][0]['value']).
      if (in_array('Drupal\Core\Entity\FieldableEntityInterface', class_implements($this->pluginDefinition['type_hint']))) {
        $filter = new FieldValueFilter($parameter->getName());

        foreach ($parameter->getFunction()->find($filter) as $lookup) {
          $lookup->replaceWith(self::rewriteFieldLookup($lookup));
        }
      }
    }
  }

  /**
   * Finds every rewritable expression in the function body.
   *
   * @param \Pharborist\Functions\ParameterNode $parameter
   *  The parameter on which the rewrite is based.
   *
   * @return \Pharborist\NodeCollection
   */
  protected function getExpressions(ParameterNode $parameter) {
    return $parameter
      ->getFunction()
      ->find(Filter::isInstanceOf('\Pharborist\Variables\VariableNode'))
      ->filter(function(VariableNode $variable) use ($parameter) {
        return $variable->getName() == $parameter->getName();
      })
      ->filter(function(VariableNode $variable) {
        $parent = $variable->parent();
        return ($parent instanceof ObjectPropertyNode || $parent instanceof ArrayLookupNode);
      })
      ->parent();
  }

  /**
   * Returns the property used by a rewritable expression, or NULL if the
   * property cannot be determined.
   *
   * @param \Pharborist\ExpressionNode $expr
   *  The rewritable expression.
   *
   * @return string|NULL
   */
  protected function getProperty(ExpressionNode $expr) {
    if ($expr instanceof ObjectPropertyNode) {
      return $expr->getPropertyName();
    }
    elseif ($expr instanceof ArrayLookupNode) {
      $key = $expr->getKey(0);

      if ($key instanceof StringNode) {
        return $key->toValue();
      }
    }
  }

  /**
   * Rewrites the given expression as a property getter. Returns NULL if the
   * expression cannot be rewritten.
   *
   * @param \Pharborist\ExpressionNode $expr
   *  The expression to rewrite.
   * @param string $property
   *  The property being used in the expression.
   *
   * @return \Pharborist\ExpressionNode|NULL
   */
  public function rewriteAsGetter(ExpressionNode $expr, $property) {
    if ($expr instanceof ObjectPropertyNode) {
      $object = clone $expr->getObject();
    }
    elseif ($expr instanceof ArrayLookupNode) {
      $object = clone $expr->getArray();
    }

    if (isset($object) && isset($this->pluginDefinition['properties'][$property]['get'])) {
      return ObjectMethodCallNode::create($object, $this->pluginDefinition['properties'][$property]['get']);
    }
  }

  /**
   * Rewrites an assignment expression as a property setter. Returns NULL if
   * the expression cannot be rewritten.
   *
   * @param \Pharborist\ExpressionNode $expr
   *  The expression to rewrite.
   * @param string $property
   *  The property being used in the expression.
   * @param \Pharborist\Operators\AssignNode $assignment
   *  The entire assignment expression being rewritten.
   *
   * @return \Pharborist\ExpressionNode|NULL
   */
  public function rewriteAsSetter(ExpressionNode $expr, $property, AssignNode $assignment) {
    if ($expr instanceof ObjectPropertyNode) {
      $object = clone $expr->getObject();
    }
    elseif ($expr instanceof ArrayLookupNode) {
      $object = clone $expr->getArray();
    }

    if (isset($object) && isset($this->pluginDefinition['properties'][$property]['set'])) {
      return ObjectMethodCallNode::create($object, $this->pluginDefinition['properties'][$property]['set'])
        ->appendArgument(clone $assignment->getRightOperand());
    }
  }

  /**
   * Returns if the parameter is fully reassigned anywhere in the function.
   *
   * @param \Pharborist\Functions\ParameterNode $parameter
   *  The parameter to check.
   *
   * @return boolean
   */
  protected function isReassigned(ParameterNode $parameter) {
    return (boolean) $parameter
      ->getFunction()
      ->find(Filter::isInstanceOf('\Pharborist\Variables\VariableNode'))
      ->filter(function(VariableNode $variable) use ($parameter) {
        return $variable->getName() == $parameter->getName();
      })
      ->filter($this->isAssigned)
      ->count();
  }

  /**
   * Rewrites a Drupal 7 field lookup like so:
   *
   * $node->body[LANGUAGE_NONE][0]['value'] --> $node->body[0]->value
   * $node->body['fr'][0]['value'] --> $node->getTranslation('fr')->body[0]->value
   *
   * @param \Pharborist\ArrayLookupNode $node
   *  The original field lookup.
   *
   * @return \Pharborist\ExpressionNode
   */
  public static function rewriteFieldLookup(ArrayLookupNode $node) {
    $keys = $node->getKeys();
    /** @var \Pharborist\Objects\ObjectPropertyNode $root */
    $root = $node->getRoot();
    $expr = $root->getObject()->getText();

    if (self::isTranslation($keys[0])) {
      $expr .= '->getTranslation(' . $keys[0] . ')';
    }
    $expr .= '->' . $root->getPropertyName() . '[' . $keys[1] . ']';

    /** @var \Pharborist\Types\StringNode|\Pharborist\Node $column */
    foreach (array_slice($keys, 2) as $column) {
      $expr .= '->';
      $expr .= $column instanceof StringNode ? $column->toValue() : $column->getText();
    }

    return Parser::parseExpression($expr);
  }

  /**
   * Checks if a field lookup key is translated. This will be TRUE unless one
   * of the following conditions applies:
   *
   * - The key is the Drupal\Core\Language\Language::LANGCODE_NOT_SPECIFIED
   *   constant.
   * - The key is the LANGUAGE_NONE constant from Drupal 7.
   * - The key is the string 'und'.
   *
   * @param Node $key
   *  The key to check.
   *
   * @return boolean
   */
  public static function isTranslation(Node $key) {
    if ($key instanceof ClassConstantLookupNode) {
      $constant = $key->getClassName() . '::' . $key->getConstantName();
      return $constant != '\Drupal\Core\Language\Language::LANGCODE_NOT_SPECIFIED';
    }
    elseif ($key instanceof ConstantNode) {
      return $key->getConstantName() != 'LANGUAGE_NONE';
    }
    elseif ($key instanceof StringNode) {
      return $key->toValue() != 'und';
    }
    else {
      return TRUE;
    }
  }

}
