<?php

namespace Drupal\drupalmoduleupgrader\Plugin\DMU\Routing;

use Drupal\Component\Plugin\PluginManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\drupalmoduleupgrader\Converter\Routing\ParameterMap;
use Drupal\drupalmoduleupgrader\ConverterBase;
use Drupal\drupalmoduleupgrader\Converter\Routing\Drupal7\RouteWrapper as Drupal7Route;
use Drupal\drupalmoduleupgrader\Converter\Routing\Drupal8\RouteWrapper as Drupal8Route;
use Drupal\drupalmoduleupgrader\Converter\Routing\RouteConverterInterface;
use Drupal\drupalmoduleupgrader\TargetInterface;
use Drupal\drupalmoduleupgrader\Utility\StringTransformTrait;
use Pharborist\Objects\ClassMethodCallNode;
use Pharborist\Filter;
use Pharborist\Functions\FunctionDeclarationNode;
use Pharborist\Functions\ParameterNode;
use Pharborist\ReturnStatementNode;
use Drupal\Core\Routing\RouteProviderInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Routing\Route as CoreRoute;

/**
 * @Converter(
 *  id = "default",
 *  description = @Translation("Converts a menu item to a _content route.")
 * )
 */
class ContentRoute extends ConverterBase implements RouteConverterInterface, ContainerFactoryPluginInterface {

  use StringTransformTrait;

  /**
   * @var \Drupal\Core\Routing\RouteProviderInterface
   */
  protected $routeProvider;

  /**
   * @var \Drupal\Component\Plugin\PluginManagerInterface
   */
  protected $rewriters;

  /**
   * Constructs a RouteConverterBase object.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, RouteProviderInterface $route_provider, PluginManagerInterface $rewriters) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->routeProvider = $route_provider;
    $this->rewriters = $rewriters;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('router.route_provider'),
      $container->get('plugin.manager.drupalmoduleupgrader.rewriter')
    );
  }

  /**
   * Conform with ConverterInterface, which we implement through ConverterBase.
   * Because route conversion is so complex, the Routing plugin never calls
   * this method. It relies instead on the other methods defined in
   * RouteConverterInterface.
   */
  final public function convert(TargetInterface $target) {}

  /**
   * {@inheritdoc}
   */
  public function getName(TargetInterface $target, Drupal7Route $route) {
    $name = $target->id() . '.' . $this->unPrefix($route['page callback'], $target->id());

    $arguments = array_filter($route['page arguments'], 'is_string');
    if ($arguments) {
      $name .= '_' . implode('_', $arguments);
    }

    return $name;
  }

  /**
   * {@inheritdoc}
   */
  public function buildPath(TargetInterface $target, Drupal7Route $route) {
    // The parameter map modifies the path in-place, so we'll clone it in order
    // to keep this method non-destructive.
    $path = clone $route->getPath();
    $this->buildParameterMap($target, $route)->applyPath($path);
    return $path;
  }

  /**
   * Builds a parameter map from the aggregated arguments of the title,
   * access, and page callbacks.
   *
   * @return \Drupal\drupalmoduleupgrader\Converter\Routing\ParameterMap
   */
  protected function buildParameterMap(TargetInterface $target, Drupal7Route $route) {
    $map = new ParameterMap(clone $route->getPath(), []);

    $indexer = $target->getIndexer('function');

    if ($indexer->has($route['title callback'])) {
      $map->merge(new ParameterMap(
        $route->getPath(),
        $indexer->get($route['title callback'])->get(0)->getParameters()->toArray(),
        $route['title arguments']
      ));
    }

    if ($indexer->has($route['access callback'])) {
      $map->merge(new ParameterMap(
        $route->getPath(),
        $indexer->get($route['access callback'])->get(0)->getParameters()->toArray(),
        $route['access arguments']
      ));
    }

    if ($indexer->has($route['page callback'])) {
      $map->merge(new ParameterMap(
        $route->getPath(),
        $indexer->get($route['page callback'])->get(0)->getParameters()->toArray(),
        $route['page arguments']
      ));
    }

    return $map;
  }

  /**
   * {@inheritdoc}
   */
  public function buildRouteDefinition(TargetInterface $target, Drupal7Route $route) {
    $indexer = $target->getIndexer('function');

    $definition = new CoreRoute('');
    $this->buildParameterMap($target, $route)->applyRoute($definition);

    $controller = $this->getController($target, $route)->getName()->getAbsolutePath();

    if ($route->containsKey('title')) {
      $definition->setDefault('_title', $route['title']);
    }
    elseif ($indexer->has($route['title callback'])) {
      $definition->setDefault('_title_callback', $controller . '::' . $route['title callback']);
    }

    if ($route->isAbsoluteAccess()) {
      $definition->setRequirement('_access', $route['access callback'] ? 'true' : 'false');
    }
    elseif ($route->isPermissionBased()) {
      $definition->setRequirement('_permission', $route['access arguments'][0]);
    }
    elseif ($indexer->has($route['access callback'])) {
      $definition->setRequirement('_custom_access', $controller . '::' . $route['access callback']);
    }

    if ($indexer->has($route['page callback'])) {
      $definition->setDefault('_content', $controller . '::' . $route['page callback']);
    }

    return new Drupal8Route($this->getName($target, $route), $definition, $this->routeProvider);
  }

  /**
   * {@inheritdoc}
   */
  public function buildRoute(TargetInterface $target, Drupal7Route $route) {
    $definition = $this->buildRouteDefinition($target, $route);

    $map = $this->buildParameterMap($target, $route);
    $map->applyRoute($definition->unwrap());

    $indexer = $target->getIndexer('function');

    foreach ($map->toArray() as $function_name => $parameters) {
      if ($parameters && $indexer->has($function_name)) {
        /** @var \Pharborist\Functions\FunctionDeclarationNode $function */
        $function = $indexer->get($function_name)->get(0);
        foreach ($parameters as $parameter_name => $info) {
          $parameter = $function->getParameterByName($parameter_name)->setName($info['name'], TRUE);
          if (isset($info['type'])) {
            $this->rewriters->createInstance('_rewriter:' . $info['type'])->rewrite($parameter);
          }
        }
      }
    }

    $class_indexer = $target->getIndexer('class');
    if ($class_indexer->has('DefaultController')) {
      $controller = $class_indexer->get('DefaultController')->get(0);
    }
    else {
      $controller = $this->getController($target, $route);
      $class_indexer->add('DefaultController', $this->writeClass($target, $controller));
    }

    if ($indexer->has($route['title callback'])) {
      if (! $controller->hasMethod($route['title callback'])) {
        $indexer->get($route['title callback'])->get(0)->cloneAsMethodOf($controller);
      }
    }

    if ($indexer->has($route['access callback'])) {
      $func = $indexer->get($route['access callback'])->get(0);

      $returns = $func->find(Filter::isInstanceOf('\Pharborist\ReturnStatementNode'));
      foreach ($returns as $ret) {
        $call = ClassMethodCallNode::create('\Drupal\Core\Access\AccessResult', 'allowedIf')->appendArgument($ret->getExpression());
        $ret->replaceWith(ReturnStatementNode::create($call));
      }

      // The access callback always receives an $account parameter.
      if ($func->hasParameter('account')) {
        $func->getParameter('account')->setTypeHint('Drupal\Core\Session\AccountInterface');
      }
      else {
        $account = ParameterNode::create('account')->setTypeHint('Drupal\Core\Session\AccountInterface');
        $func->appendParameter($account);
      }

      if (! $controller->hasMethod($route['access callback'])) {
        $func->cloneAsMethodOf($controller);
      }
    }

    if ($indexer->has($route['page callback'])) {
      if (! $controller->hasMethod($route['page callback'])) {
        $indexer->get($route['page callback'])->get(0)->cloneAsMethodOf($controller);
      }
    }

    $this->writeClass($target, $controller);
  }

  protected function getController(TargetInterface $target, Drupal7Route $route) {
    $render = [
      '#theme' => 'dmu_controller',
      '#module' => $target->id(),
    ];
    return $this->parse($render);
  }

}
