<?php

namespace Drupal\drupalmoduleupgrader\Tests;

use Drupal\drupalmoduleupgrader\Converter\Routing\Drupal7\RouteWrapper as D7Route;
use Drupal\drupalmoduleupgrader\Converter\Routing\Drupal8\RouteWrapper;
use Drupal\drupalmoduleupgrader\Target;
use Drupal\drupalmoduleupgrader\Utility\Path\PathUtilityInterface;
use Drupal\simpletest\KernelTestBase;
use Pharborist\ClassNode;
use Pharborist\Filter;
use Pharborist\TopNode;
use Symfony\Component\Filesystem\Filesystem;

/**
 * Tests converting a hook_menu() item pointing to a custom callback into a
 * method of a generic controller.
 *
 * @group DMU
 */
class ContentRouteTest extends KernelTestBase {

  public static $modules = ['user', 'drupalmoduleupgrader'];

  /**
   * @var \Drupal\drupalmoduleupgrader\Converter\Routing\Drupal7\RouteWrapper
   */
  protected $wrapper;

  /**
   * @var \Symfony\Component\Filesystem\Filesystem
   */
  protected $fs;

  /**
   * @var \Drupal\drupalmoduleupgrader\TargetInterface
   */
  protected $target;

  /**
   * @var \Drupal\drupalmoduleupgrader\Converter\Routing\RouteConverterInterface
   */
  protected $plugin;

  public function setUp() {
    parent::setUp();
    $this->installSchema('drupalmoduleupgrader', 'dmu_index');

    $route = [
      'title' => 'Change pants',
      'page callback' => 'pants_change',
      'page arguments' => array(2),
      'access callback' => 'pants_change_access', // Default.
      'delivery callback' => 'ajax_deliver',
      'type' => 'MENU_CALLBACK',
      'file' => 'pants.pages.inc',
    ];
    $this->wrapper = new D7Route('pants/change/%user', $route);

    $source_path = drupal_get_path('module', 'drupalmoduleupgrader') . '/tests/pants';
    $this->fs = new Filesystem();
    $this->target = new Target($source_path, \Drupal::getContainer());
    $this->plugin = \Drupal::service('plugin.manager.drupalmoduleupgrader.route')->createInstance('default', [ 'route' => $this->wrapper ]);
  }

  public function tearDown() {
    $this->fs->remove($this->target->getPath('src'));
  }

  public function testGetName() {
    $name = $this->plugin->getName($this->target, $this->wrapper);
    $this->assertEqual('pants.change', $name);
  }

  public function testBuildPath() {
    $path = $this->plugin->buildPath($this->target, $this->wrapper);
    $this->assertTrue($path instanceof PathUtilityInterface);
    $this->assertEqual('pants/change/{account}', (string) $path);
  }

  public function testBuildRouteDefinition() {
    $definition = $this->plugin->buildRouteDefinition($this->target, $this->wrapper);

    $this->assertTrue($definition instanceof RouteWrapper);
    $this->assertTrue($definition->hasDefault('_title'));
    $this->assertEqual('Change pants', $definition->getDefault('_title'));
    $this->assertTrue($definition->hasRequirement('_custom_access'));
    $this->assertEqual('\Drupal\pants\Controller\DefaultController::pants_change_access', $definition->getRequirement('_custom_access'));
    $this->assertTrue($definition->hasDefault('_content'));
    $this->assertEqual('\Drupal\pants\Controller\DefaultController::pants_change', $definition->getDefault('_content'));
  }

  public function testBuildRoute() {
    $this->plugin->buildRoute($this->target, $this->wrapper);

    $controller_file = $this->target->getPath('src/Controller/DefaultController.php');
    $this->assertTrue(file_exists($controller_file));

    /** @var \Pharborist\ClassNode $controller */
    $controller = $this->target
      ->getCodeManager()
      ->open($controller_file)
      ->find(Filter::isClass('DefaultController'))
      ->get(0);
    $this->assertTrue($controller->hasMethod('pants_change_access'));
    $this->assertTrue($controller->hasMethod('pants_change'));
  }

}
