<?php

namespace Drupal\drupalmoduleupgrader\Tests;

use Drupal\drupalmoduleupgrader\Converter\Routing\Drupal7\RouteWrapper as D7Route;
use Drupal\drupalmoduleupgrader\Converter\Routing\Drupal8\RouteWrapper as D8Route;
use Drupal\drupalmoduleupgrader\Converter\Routing\Drupal8\RouteWrapper;
use Drupal\drupalmoduleupgrader\Target;
use Drupal\simpletest\KernelTestBase;
use Pharborist\ClassNode;
use Pharborist\Filter;
use Pharborist\TopNode;
use Symfony\Component\Filesystem\Filesystem;

/**
 * Tests converting a hook_menu() item that calls drupal_get_form into a
 * class implemeting FormInterface.
 *
 * @group DMU
 */
class FormRouteTest extends KernelTestBase {

  public static $modules = ['drupalmoduleupgrader'];

  /**
   * @var \Drupal\drupalmoduleupgrader\Converter\Routing\Drupal7\RouteWrapper
   */
  protected $wrapper;

  /**
   * @var \Symfony\Component\Filesystem\Filesystem
   */
  protected $fs;

  /**
   * @var \Drupal\drupalmoduleupgrader\TargetInterface
   */
  protected $target;

  /**
   * @var \Drupal\drupalmoduleupgrader\Converter\Routing\RouteConverterInterface
   */
  protected $plugin;

  public function setUp() {
    parent::setUp();
    $this->installSchema('drupalmoduleupgrader', 'dmu_index');

    $route = [
      'title' => 'Pants',
      'description' => 'Administer pants.',
      'page callback' => 'drupal_get_form',
      'page arguments' => array('pants_settings'),
      'access callback' => 'user_access',
      'access arguments' => array('administer pants'),
      'type' => 'MENU_NORMAL_ITEM',
      'file' => 'pants.admin.inc',
    ];
    $this->wrapper = new D7Route('admin/config/people/pants', $route);

    $source_path = drupal_get_path('module', 'drupalmoduleupgrader') . '/tests/pants';
    $this->fs = new Filesystem();
    $this->target = new Target($source_path, \Drupal::getContainer());
    $this->plugin = \Drupal::service('plugin.manager.drupalmoduleupgrader.route')->createInstance('drupal_get_form');
  }

  public function tearDown() {
    $this->fs->remove($this->target->getPath('src'));
  }

  public function testGetName() {
    $this->assertEqual('pants.settings', $this->plugin->getName($this->target, $this->wrapper));
  }

  public function testBuildRouteDefinition() {
    $definition = $this->plugin->buildRouteDefinition($this->target, $this->wrapper);

    $this->assertTrue($definition instanceof RouteWrapper);
    $this->assertTrue($definition->hasDefault('_title'));
    $this->assertEqual('Pants', $definition->getDefault('_title'));
    $this->assertTrue($definition->hasRequirement('_permission'));
    $this->assertEqual('administer pants', $definition->getRequirement('_permission'));
    $this->assertTrue($definition->hasDefault('_form'));
    $this->assertEqual('\Drupal\pants\Form\PantsSettings', $definition->getDefault('_form'));
    // FormRoute extends ContentRoute, so we don't want any values from that
    // to bleed through.
    $this->assertFalse($definition->hasDefault('_content'));
  }

  public function testBuildPath() {
    $path = $this->plugin->buildPath($this->target, $this->wrapper);
    $this->assertEqual('admin/config/people/pants', (string) $path);
  }

  public function testBuildRoute() {
    $this->plugin->buildRoute($this->target, $this->wrapper);

    $form_file = $this->target->getPath('src/Form/PantsSettings.php');
    $this->assertTrue(file_exists($form_file));

    /** @var \Pharborist\ClassNode $class */
    $class = $this->target
      ->getCodeManager()
      ->open($form_file)
      ->find(Filter::isClass('PantsSettings'))
      ->get(0);
    $this->assertTrue($class instanceof ClassNode);
    $this->assertTrue($class->hasMethod('buildForm'));
    $this->assertTrue($class->hasMethod('submitForm'));
  }

}
